@@ -7,12 +7,15 @@ import sys
 import tempfile
 from shutil import rmtree
 
-version = '21w16 dev'
+version = '21w19a'
+
+def error(message):
+    print('Error!', message, file=sys.stderr)
+    sys.exit(1)
 
 def file_type(file: str) -> str:
     if(not os.path.isfile(file)):
-        print('Auto-Editor could not find the file: ' + file, file=sys.stderr)
-        sys.exit(1)
+        error('Auto-Editor could not find the file: ' + file)
     return file
 
 def float_type(num: str) -> float:
@@ -38,11 +41,27 @@ def frame_type(inp: str):
         return inp[:-4]
     return int(inp)
 
-def comma_type(inp: str) -> list:
+def comma_type(inp: str, min_args=1, max_args=None, name='') -> list:
     from usefulFunctions import cleanList
-    ms = inp.split(',')
-    ms = cleanList(ms, '\r\n\t')
-    return ms
+    inp = cleanList(inp.split(','), '\r\n\t')
+    if(min_args > len(inp)):
+        error(f'Too few comma arguments for {name}.')
+    if(max_args is not None and len(inp) > max_args):
+        error(f'Too many comma arguments for {name}.')
+    return inp
+
+def zoom_type(inp):
+    return comma_type(inp, 3, 8, 'zoom_type')
+
+def rect_type(inp):
+    return comma_type(inp, 6, 8, 'rect_type')
+
+def range_type(inp):
+    return comma_type(inp, 2, 2, 'range_type')
+
+def speed_range_type(inp):
+    return comma_type(inp, 3, 3, 'speed_range_type')
+
 
 def appendFileName(file_name, val):
     dotIndex = file_name.rfind('.')
@@ -92,14 +111,15 @@ def main_options():
         help='manually set the number of tracks auto-editor thinks there are.')
 
     ops += add_argument('exportMediaOps', nargs=0, action='grouping')
-    ops += add_argument('--video_bitrate', '-vb', group='exportMediaOps',
+    ops += add_argument('--video_bitrate', '-vb', default='unset', group='exportMediaOps',
         help='set the number of bits per second for video.')
-    ops += add_argument('--audio_bitrate', '-ab', group='exportMediaOps',
+    ops += add_argument('--audio_bitrate', '-ab', default='unset', group='exportMediaOps',
         help='set the number of bits per second for audio.')
+    # ops += add_argument('--video_quality', '-')
     ops += add_argument('--sample_rate', '-r', type=sample_rate_type,
         group='exportMediaOps',
         help='set the sample rate of the input and output videos.')
-    ops += add_argument('--video_codec', '-vcodec', default='uncompressed',
+    ops += add_argument('--video_codec', '-vcodec', default='copy',
         group='exportMediaOps',
         help='set the video codec for the output media file.')
     ops += add_argument('--audio_codec', '-acodec', group='exportMediaOps',
@@ -110,7 +130,7 @@ def main_options():
         help='set the preset for ffmpeg to help save file size or increase quality.')
     ops += add_argument('--tune', '-t', default='none', group='exportMediaOps',
         choices=['film', 'animation', 'grain', 'stillimage', 'fastdecode',
-            'zerolatency', 'none'],
+            'zerolatency', 'none', 'unset'],
         help='set the tune for ffmpeg to compress video better in certain circumstances.')
     ops += add_argument('--constant_rate_factor', '-crf', type=int, default=15,
         group='exportMediaOps', range='0 to 51',
@@ -137,33 +157,38 @@ def main_options():
         help='export as an XML file for Final Cut Pro instead of outputting a media file.')
     ops += add_argument('--export_as_json', action='store_true',
         help='export as a JSON file that can be read by auto-editor later.')
-    ops += add_argument('--export_as_clip_sequence', action='store_true',
+    ops += add_argument('--export_as_clip_sequence', '-excs', action='store_true',
         help='export as multiple numbered media files.')
 
     ops += add_argument('--render', default='auto', choices=['av', 'opencv', 'auto'],
         help='choice which method to render video.')
     ops += add_argument('--scale', type=float_type, default=1,
-        help='scale the output media file by a certian size (change resolution by a factor)',
-        extra='Hello')
+        help='scale the output media file by a certian factor.')
 
-    ops += add_argument('--zoom', type=comma_type, nargs='*',
+    ops += add_argument('--zoom', type=zoom_type, nargs='*',
         help='set when and how a zoom will occur.',
         extra='The arguments are: start,end,start_zoom,end_zoom,x,y,inter,hold' \
-            '\nThere must be at least 4 comma args. x and y default to centerX and centerY' \
+            '\nThere must be at least 3 comma args. x and y default to centerX and centerY' \
             '\nThe default interpolation is linear.')
-    ops += add_argument('--rectangle', type=comma_type, nargs='*',
+    ops += add_argument('--rectangle', type=rect_type, nargs='*',
         help='overlay a rectangle shape on the video.',
         extra='The arguments are: start,end,x1,y1,x2,y2,color,thickness' \
             '\nThere must be at least 6 comma args. The rectangle is solid if' \
             ' thickness is not defined.\n The default color is #000.')
 
     ops += add_argument('--background', type=str, default='#000',
-        help='set the color of the background when the video is moved.')
+        help='set the color of the background that is visible when the video is moved.')
+
+    ops += add_argument('--mark_as_loud', type=range_type, nargs='*',
+        help='the range that will be marked as "loud".')
+    ops += add_argument('--mark_as_silent', type=range_type, nargs='*',
+        help='the range that will be marked as "silent".')
+    ops += add_argument('--cut_out', type=range_type, nargs='*',
+        help='the range of media that will be removed completely, regardless of the value of silent speed.')
+    ops += add_argument('--set_speed_for_range', type=speed_range_type, nargs='*',
+        help='set an arbitrary speed for a given range.',
+        extra='The arguments are: speed,start,end')
 
-    ops += add_argument('--ignore', nargs='*',
-        help='the range that will be marked as "loud"')
-    ops += add_argument('--cut_out', nargs='*',
-        help='the range that will be marked as "silent"')
     ops += add_argument('--motion_threshold', type=float_type, default=0.02,
         range='0 to 1',
         help='how much motion is required to be considered "moving"')
@@ -213,10 +238,11 @@ def main_options():
         range='0 to 1',
         help='set the volume that frames audio needs to surpass to be "loud".')
     ops += add_argument('--silent_speed', '-s', type=float_type, default=99999,
-        range='0 to 99999',
+        range='Any number. Values <= 0 or >= 99999 will be cut out.',
         help='set the speed that "silent" sections should be played at.')
     ops += add_argument('--video_speed', '--sounded_speed', '-v', type=float_type,
-        default=1.00, range='0 to 999999',
+        default=1.00,
+        range='Any number. Values <= 0 or >= 99999 will be cut out.',
         help='set the speed that "loud" sections should be played at.')
     ops += add_argument('--frame_margin', '-m', type=frame_type, default=6,
         range='0 to Infinity',
@@ -251,14 +277,14 @@ def main():
         sys.exit()
 
     from vanparse import ParseOptions
-    from usefulFunctions import Log, Timer
+    from usefulFunctions import Log, Timer, isLatestVersion, fNone, sep
+    from ffwrapper import FFmpeg, FFprobe
 
     subcommands = ['create', 'test', 'info', 'levels']
 
     if(len(sys.argv) > 1 and sys.argv[1] in subcommands):
         if(sys.argv[1] == 'create'):
             from create import create, create_options
-            from usefulFunctions import FFmpeg
             args = ParseOptions(sys.argv[2:], Log(), 'create', create_options())
 
             ffmpeg = FFmpeg(dirPath, args.my_ffmpeg, True, Log())
@@ -271,8 +297,6 @@ def main():
 
         if(sys.argv[1] == 'info'):
             from info import getInfo, info_options
-            from usefulFunctions import FFmpeg, FFprobe
-
             args = ParseOptions(sys.argv[2:], Log(), 'info', info_options())
 
             log = Log()
@@ -281,7 +305,6 @@ def main():
             getInfo(args.input, ffmpeg, ffprobe, args.fast, log)
         if(sys.argv[1] == 'levels'):
             from levels import levels, levels_options
-            from usefulFunctions import FFmpeg, FFprobe
             args = ParseOptions(sys.argv[2:], Log(), 'levels', levels_options())
 
             TEMP = tempfile.mkdtemp()
@@ -294,11 +317,10 @@ def main():
         option_data = main_options()
         args = ParseOptions(sys.argv[1:], Log(True), 'auto-editor', option_data)
 
-    timer = Timer(args.quiet)
-
     del option_data
 
-    from usefulFunctions import FFmpeg, FFprobe, sep
+    timer = Timer(args.quiet)
+
     ffmpeg = FFmpeg(dirPath, args.my_ffmpeg, args.show_ffmpeg_debug, Log())
     ffprobe = FFprobe(dirPath, args.my_ffmpeg, args.show_ffmpeg_debug, Log())
 
@@ -317,10 +339,7 @@ def main():
         print('Platform:', platform.system(), platform.release())
         print('Config File path:', dirPath + sep() + 'config.txt')
         print('FFmpeg path:', ffmpeg.getPath())
-        ffmpegVersion = ffmpeg.pipe(['-version']).split('\n')[0]
-        ffmpegVersion = ffmpegVersion.replace('ffmpeg version', '').strip()
-        ffmpegVersion = ffmpegVersion.split(' ')[0]
-        print('FFmpeg version:', ffmpegVersion)
+        print('FFmpeg version:', ffmpeg.getVersion())
         print('Auto-Editor version', version)
         sys.exit()
 
@@ -339,8 +358,6 @@ def main():
     ffmpeg.updateLog(log)
     ffprobe.updateLog(log)
 
-    from usefulFunctions import isLatestVersion
-
     if(not args.quiet and not isLatestVersion(version, log)):
         log.print('\nAuto-Editor is out of date. Run:\n')
         log.print('    pip3 install -U auto-editor')
@@ -386,6 +403,13 @@ def main():
         inputList = [f'{TEMP}{sep()}combined.mp4']
 
     speeds = [args.silent_speed, args.video_speed]
+    if(args.cut_out != [] and 99999 not in speeds):
+        speeds.append(99999)
+
+    for item in args.set_speed_for_range:
+        if(item[0] not in speeds):
+            speeds.append(float(item[0]))
+
     log.debug(f'   - Speeds: {speeds}')
 
     from wavfile import read
@@ -456,7 +480,7 @@ def main():
             else:
                 tracks = args.force_tracks_to
             cmd = ['-i', INPUT_FILE]
-            if(audioBitrate is not None):
+            if(not fNone(audioBitrate)):
                 cmd.extend(['-b:a', audioBitrate])
             cmd.extend(['-ac', '2', '-ar', sampleRate, '-vn', f'{TEMP}{sep()}fastAud.wav'])
             ffmpeg.run(cmd)
@@ -474,8 +498,7 @@ def main():
                 fps = ffprobe.getFrameRate(INPUT_FILE)
 
             if(fps < 1):
-                log.error(f"Frame rate cannot be below 1. fps: {fps}" \
-                    f"\nInput file: {INPUT_FILE}")
+                log.error(f"{INPUT_FILE}: Frame rate cannot be below 1. fps: {fps}")
 
             tracks = args.force_tracks_to
             if(tracks is None):
@@ -493,15 +516,31 @@ def main():
                 log.error("You choose a track that doesn't exist.\n" \
                     f'There {message}.\n {allTracks}')
 
+            def NumberOfVrfFrames(text, log):
+                import re
+                search = re.search(r'VFR:[\d.]+ \(\d+\/\d+\)', text, re.M)
+                if(search is None):
+                    log.warning('Could not get number of VRF Frames.')
+                    return 0
+                else:
+                    nums = re.search(r'\d+\/\d+', search.group()).group(0)
+                    log.debug(nums)
+                    return int(nums.split('/')[0])
+
+            def hasVFR(cmd, log):
+                return NumberOfVrfFrames(ffmpeg.pipe(cmd), log) != 0
+
             # Split audio tracks into: 0.wav, 1.wav, etc.
+            cmd = ['-i', INPUT_FILE, '-hide_banner']
             for trackNum in range(tracks):
-                cmd = ['-i', INPUT_FILE]
-                if(audioBitrate is not None):
+                cmd.extend(['-map', f'0:a:{trackNum}'])
+                if(not fNone(audioBitrate)):
                     cmd.extend(['-ab', audioBitrate])
-                cmd.extend(['-ac', '2', '-ar', sampleRate, '-map',
-                    f'0:a:{trackNum}', f'{TEMP}{sep()}{trackNum}.wav'])
-                ffmpeg.run(cmd)
-                del cmd
+                cmd.extend(['-ac', '2', '-ar', sampleRate,
+                    f'{TEMP}{sep()}{trackNum}.wav'])
+            cmd.extend(['-map', '0:v:0', '-vf', 'vfrdet', '-f', 'null', '-'])
+            has_vfr = hasVFR(cmd, log)
+            del cmd
 
             # Check if the `--cut_by_all_tracks` flag has been set or not.
             if(args.cut_by_all_tracks):
@@ -560,25 +599,25 @@ def main():
                 from cutting import applyRects
                 effects += applyRects(args.rectangle, audioData, sampleRate, fps, log)
 
-            chunks = applySpacingRules(hasLoud, fps, args.frame_margin,
-                args.min_clip_length, args.min_cut_length, args.ignore, args.cut_out, log)
+            chunks = applySpacingRules(hasLoud, speeds, fps, args, log)
             del hasLoud
 
-        clips = []
-        numCuts = 0
-        for chunk in chunks:
-            if(speeds[chunk[2]] != 99999):
-                clips.append([chunk[0], chunk[1], speeds[chunk[2]] * 100])
-            else:
-                numCuts += 1
+        def isClip(chunk):
+            nonlocal speeds
+            return speeds[chunk[2]] != 99999
 
-        if(fps is None and not audioFile):
-            if(makingDataFile):
-                constantLoc = appendFileName(INPUT_FILE, '_constantFPS')
-            else:
-                constantLoc = f'{TEMP}{sep()}constantVid{fileFormat}'
-            ffmpeg.run(['-i', INPUT_FILE, '-filter:v', 'fps=fps=30', constantLoc])
-            INPUT_FILE = constantLoc
+        def getNumberOfCuts(chunks, speeds):
+            return len(list(filter(isClip, chunks)))
+
+        def getClips(chunks, speeds):
+            clips = []
+            for chunk in chunks:
+                if(isClip(chunk)):
+                    clips.append([chunk[0], chunk[1], speeds[chunk[2]] * 100])
+            return clips
+
+        numCuts = getNumberOfCuts(chunks, speeds)
+        clips = getClips(chunks, speeds)
 
         if(args.export_as_json):
             from makeCutList import makeCutList
@@ -607,10 +646,11 @@ def main():
             from fastAudio import fastAudio, handleAudio, convertAudio
             theFile = handleAudio(ffmpeg, input_, audioBitrate, str(sampleRate),
                 TEMP, log)
-            fastAudio(theFile, f'{TEMP}{sep()}convert.wav', chunks, speeds, log, fps,
+
+            TEMP_FILE = f'{TEMP}{sep()}convert.wav'
+            fastAudio(theFile, TEMP_FILE, chunks, speeds, log, fps,
                 args.machine_readable_progress, args.no_progress)
-            convertAudio(ffmpeg, ffprobe, f'{TEMP}{sep()}convert.wav', input_,
-                output, args, log)
+            convertAudio(ffmpeg, ffprobe, TEMP_FILE, input_, output, args, log)
 
         if(audioFile):
             if(args.export_as_clip_sequence):
@@ -648,12 +688,12 @@ def main():
                         log.error('Rectangle effect is not supported on the av render method.')
 
                     from renderVideo import renderAv
-                    renderAv(ffmpeg, ffprobe, input_, args, chunks, speeds, fps,
+                    renderAv(ffmpeg, ffprobe, input_, args, chunks, speeds, fps, has_vfr,
                     TEMP, log)
 
                 if(args.render == 'opencv'):
                     from renderVideo import renderOpencv
-                    renderOpencv(ffmpeg, ffprobe, input_, args, chunks, speeds, fps,
+                    renderOpencv(ffmpeg, ffprobe, input_, args, chunks, speeds, fps, has_vfr,
                         effects, TEMP, log)
 
                 # Now mix new audio(s) and the new video.
@@ -665,11 +705,11 @@ def main():
             i = 1
             totalFrames = chunks[len(chunks) - 1][1]
             speeds.append(99999) # guarantee we have a cut speed to work with.
-            for item in chunks:
-                if(speeds[item[2]] == 99999):
+            for chunk in chunks:
+                if(speeds[chunk[2]] == 99999):
                     continue
 
-                makeVideoFile(INPUT_FILE, padChunk(item, totalFrames),
+                makeVideoFile(INPUT_FILE, padChunk(chunk, totalFrames),
                     appendFileName(newOutput, f'-{i}'))
                 i += 1
         else:
@@ -688,8 +728,8 @@ def main():
         log.print(f', which would have taken about {timeSave} if edited manually.')
 
     if(not args.no_open):
-        from usefulFunctions import smartOpen
-        smartOpen(newOutput, log)
+        from usefulFunctions import openWithSystemDefault
+        openWithSystemDefault(newOutput, log)
 
     log.debug('Deleting temp dir')
 
