@@ -197,32 +197,35 @@ def removeSmall(hasLoud: np.ndarray, lim: int, replace: bool, with_: bool) -> np
     return hasLoud
 
 
-def setRange(includeFrame: np.ndarray, syntaxRange, fps: float, with_: bool, log) -> np.ndarray:
-    end = len(includeFrame) - 1
+def isNumber(val):
+    return val.replace('.','',1).isdigit()
+
+def setRange(hasLoud: np.ndarray, syntaxRange, fps: float, with_, log) -> np.ndarray:
+
+    def replaceVarsWithVals(item) -> int:
+        nonlocal hasLoud
+        nonlocal log
+
+        if(isNumber(item)):
+            return int(item)
+        if(item == 'start'):
+            return 0
+        if(item == 'end'):
+            return len(hasLoud)
+        if(item.startswith('sec')):
+            log.error('Seconds unit not implemented in this function.')
+        log.error(f'Variable {item} not avaiable in this context.')
+
+    def ConvertToFrames(num) -> int:
+        num = replaceVarsWithVals(num)
+        if(num < 0):
+            num = len(hasLoud) - num
+        return num
+
     for item in syntaxRange:
-        pair = []
-
-        if(item.count('-') > 1):
-            log.error('Too many deliminators!')
-        if(item.count('-') < 1):
-            log.error('Invalid range. Use range syntax. ex: 5-10')
-
-        for num in item.split('-'):
-            if(num == 'start'):
-                pair.append(0)
-            elif(num == 'end'):
-                pair.append(end)
-            elif(float(num) < 0):
-                # Negative numbers = frames from end.
-                value = end - round(float(num) * fps)
-                if(value < 0):
-                    value = 0
-                pair.append(value)
-                del value
-            else:
-                pair.append(round(float(num) * fps))
-        includeFrame[pair[0]:pair[1]+1] = with_
-    return includeFrame
+        pair = list(map(ConvertToFrames, item))
+        hasLoud[pair[0]:pair[1]] = with_
+    return hasLoud
 
 
 def secToFrames(value, fps):
@@ -232,70 +235,73 @@ def secToFrames(value, fps):
 
 def cook(hasLoud: np.ndarray, minClip: int, minCut: int) -> np.ndarray:
     # Remove small loudness spikes
-    hasLoud = removeSmall(hasLoud, minClip, replace=True, with_=False)
+    hasLoud = removeSmall(hasLoud, minClip, replace=1, with_=0)
     # Remove small silences
-    hasLoud = removeSmall(hasLoud, minCut, replace=False, with_=True)
+    hasLoud = removeSmall(hasLoud, minCut, replace=0, with_=1)
     return hasLoud
 
 
 # Turn long silent/loud array to formatted chunk list.
 # Example: [True, True, True, False, False] => [[0, 3, 1], [3, 5, 0]]
-def chunkify(includeFrame, arrayLen=None) -> list:
-    if(arrayLen is None):
-        arrayLen = len(includeFrame)
+def chunkify(hasLoud, hasLoudLengthCache=None) -> list:
+    if(hasLoudLengthCache is None):
+        hasLoudLengthCache = len(hasLoud)
 
     chunks = []
     startP = 0
-    for j in range(1, arrayLen):
-        if(includeFrame[j] != includeFrame[j - 1]):
-            chunks.append([startP, j, int(includeFrame[j-1])])
+    for j in range(1, hasLoudLengthCache):
+        if(hasLoud[j] != hasLoud[j - 1]):
+            chunks.append([startP, j, int(hasLoud[j-1])])
             startP = j
-    chunks.append([startP, arrayLen, int(includeFrame[j])])
+    chunks.append([startP, hasLoudLengthCache, int(hasLoud[j])])
     return chunks
 
 
-def applySpacingRules(hasLoud: np.ndarray, fps: float, frameMargin: int,
-    minClip: int, minCut: int, ignore, cutOut, log):
-
-    frameMargin = secToFrames(frameMargin, fps)
-    minClip = secToFrames(minClip, fps)
-    minCut = secToFrames(minCut, fps)
-
-    log.checkType(frameMargin, 'frameMargin', int)
+def applySpacingRules(hasLoud: np.ndarray, speeds, fps: float, args, log) -> list:
+    frameMargin = secToFrames(args.frame_margin, fps)
+    minClip = secToFrames(args.min_clip_length, fps)
+    minCut = secToFrames(args.min_cut_length, fps)
 
     hasLoud = cook(hasLoud, minClip, minCut)
+    hasLoudLengthCache = len(hasLoud)
 
-    arrayLen = len(hasLoud)
+    def applyFrameMargin(hasLoud, hasLoudLengthCache, frameMargin) -> np.ndarray:
+        new = np.zeros((hasLoudLengthCache), dtype=np.uint8)
+        for i in range(hasLoudLengthCache):
+            start = int(max(0, i - frameMargin))
+            end = int(min(hasLoudLengthCache, i+1+frameMargin))
+            new[i] = min(1, np.max(hasLoud[start:end]))
+        return new
 
-    # Apply frame margin rules.
-    includeFrame = np.zeros((arrayLen), dtype=np.bool_)
-    for i in range(arrayLen):
-        start = int(max(0, i - frameMargin))
-        end = int(min(arrayLen, i+1+frameMargin))
-        includeFrame[i] = min(1, np.max(hasLoud[start:end]))
+    hasLoud = applyFrameMargin(hasLoud, hasLoudLengthCache, frameMargin)
 
-    del hasLoud
+    if(args.mark_as_loud != []):
+        hasLoud = setRange(hasLoud, args.mark_as_loud, fps, 1, log)
 
-    # Apply ignore rules if applicable.
-    if(ignore != []):
-        includeFrame = setRange(includeFrame, ignore, fps, True, log)
-
-    # Cut out ranges.
-    if(cutOut != []):
-        includeFrame = setRange(includeFrame, cutOut, fps, False, log)
+    if(args.mark_as_silent != []):
+        hasLoud = setRange(hasLoud, args.mark_as_silent, fps, 0, log)
 
     # Remove small clips/cuts created by applying other rules.
-    includeFrame = cook(includeFrame, minClip, minCut)
+    hasLoud = cook(hasLoud, minClip, minCut)
 
-    return chunkify(includeFrame, arrayLen)
+    if(args.cut_out != []):
+        cut_speed_index = speeds.index(99999)
+        hasLoud = setRange(hasLoud, args.cut_out, fps, cut_speed_index, log)
+
+    if(args.set_speed_for_range != []):
+        for item in args.set_speed_for_range:
+            my_speed_index = speeds.index(float(item[0]))
+            hasLoud = setRange(hasLoud, [item[1:]], fps, my_speed_index, log)
+
+    return chunkify(hasLoud, hasLoudLengthCache)
 
 
 def applyBasicSpacing(hasLoud: np.ndarray, fps: float, minClip: int, minCut: int, log):
     minClip = secToFrames(minClip, fps)
     minCut = secToFrames(minCut, fps)
 
-    includeFrame = cook(hasLoud, minClip, minCut)
-    return chunkify(includeFrame)
+    hasLoud = cook(hasLoud, minClip, minCut)
+    return chunkify(hasLoud)
 
 
 def merge(start_list: np.ndarray, end_list: np.ndarray) -> np.ndarray:
@@ -396,10 +402,6 @@ def applyRects(cmdRects, audioData, sampleRate, fps, log):
 def applyZooms(cmdZooms, audioData, sampleRate, fps, log):
     zooms = []
     for ms in cmdZooms:
-        if(len(ms) < 3):
-            # Add more description later!
-            log.error('Too few comma arguments for zoom option.' \
-                " Make sure you're wrapping the values in quotes.")
 
         start, end = ms[:2]
 
@@ -424,9 +426,6 @@ def applyZooms(cmdZooms, audioData, sampleRate, fps, log):
         if(len(ms) > 7):
             hold = ms[7]
 
-        if(len(ms) > 8):
-            log.error('Too many comma arguments for zoom option.')
-
         start_list, end_list = None, None
         if(start.startswith('audio')):
             start_list = handleBoolExp(start, audioData, sampleRate, fps, log)
