@@ -1,30 +1,33 @@
 '''fastAudio.py'''
 
-from usefulFunctions import ProgressBar, getNewLength, sep
+from usefulFunctions import ProgressBar, getNewLength, sep, fNone
 
 def convertAudio(ffmpeg, ffprobe, theFile, INPUT_FILE, outFile, args, log):
     log.debug(f'Convering internal audio file: {theFile} to {outFile}')
 
     realCodec = args.audio_codec
-    if(realCodec is None):
+    if(fNone(realCodec)):
         realCodec = ffprobe.getAudioCodec(INPUT_FILE)
-    if(realCodec == "pcm_s16le" and outFile.endswith('.m4a')):
+    if(realCodec == 'pcm_s16le' and outFile.endswith('.m4a')):
         log.error(f'Codec: {realCodec} is not supported in the m4a container.')
 
     ffmpeg.run(['-i', theFile, '-acodec', realCodec, outFile])
 
 
 def handleAudio(ffmpeg, theFile, audioBit, samplerate: str, temp, log) -> str:
+    TEMPFILE = f'{temp}{sep()}faAudio.wav'
+
     log.checkType(samplerate, 'samplerate', str)
     cmd = ['-i', theFile]
-    if(audioBit is not None):
+    if(not fNone(audioBit)):
         cmd.extend(['-b:a', audioBit])
         log.checkType(audioBit, 'audioBit', str)
-    cmd.extend(['-ac', '2', '-ar', samplerate, '-vn', f'{temp}{sep()}faAudio.wav'])
+    cmd.extend(['-ac', '2', '-ar', samplerate, '-vn', TEMPFILE])
+
     ffmpeg.run(cmd)
     log.conwrite('')
 
-    return f'{temp}{sep()}faAudio.wav'
+    return TEMPFILE
 
 
 def fastAudio(theFile, outFile, chunks: list, speeds: list, log, fps: float,
@@ -34,15 +37,18 @@ def fastAudio(theFile, outFile, chunks: list, speeds: list, log, fps: float,
 
     import numpy as np
 
-    if((speeds[0] != 1 and speeds[0] != 99999) or (speeds[1] != 1 and speeds[1] != 99999)):
-        from audiotsm2 import phasevocoder
-        from audiotsm2.io.array import ArrReader, ArrWriter
-
     log.checkType(chunks, 'chunks', list)
     log.checkType(speeds, 'speeds', list)
 
+    def speedsOtherThan1And99999(a: list) -> bool:
+        return len([x for x in a if x != 1 and x != 99999]) > 0
+
+    if(speedsOtherThan1And99999(speeds)):
+        from audiotsm2 import phasevocoder
+        from audiotsm2.io.array import ArrReader, ArrWriter
+
     if(len(chunks) == 1 and chunks[0][2] == 0):
-        log.error('Trying to create empty audio.')
+        log.error('Trying to create an empty file.')
 
     if(not os.path.isfile(theFile)):
         log.error('fastAudio.py could not find file: ' + theFile)
@@ -101,3 +107,5 @@ def fastAudio(theFile, outFile, chunks: list, speeds: list, log, fps: float,
     log.debug('   - Expected video length: ' + str(yPointer / (samplerate / fps)))
     newAudio = newAudio[:yPointer]
     write(outFile, samplerate, newAudio)
+
+    log.conwrite('')
