@@ -1,39 +1,89 @@
 '''renderVideo.py'''
 
-# Included Libaries
-from usefulFunctions import ProgressBar, sep
-
 # Internal Libraries
 import subprocess
+from os import path
+from platform import system
+
+# Included Libaries
+from usefulFunctions import ProgressBar, sep, fNone
 
 def properties(cmd, args, vidFile, ffprobe):
-    if(args.video_codec == 'copy'):
-        cmd.extend(['-vcodec', ffprobe.getVideoCodec(vidFile)])
-    elif(args.video_codec == 'uncompressed'):
+
+    def fset(cmd, option, value):
+        if(fNone(value)):
+            return cmd
+        return cmd + [option] + [value]
+
+    if(args.video_codec == 'uncompressed'):
         cmd.extend(['-vcodec', 'mpeg4', '-qscale:v', '1'])
+    elif(args.video_codec == 'copy'):
+        new_codec = ffprobe.getVideoCodec(vidFile)
+        if(new_codec != 'dvvideo'): # This codec seems strange.
+            cmd.extend(['-vcodec', new_codec])
     else:
-        cmd.extend(['-vcodec', args.video_codec])
+        cmd = fset(cmd, '-vcodec', args.video_codec)
 
-        if(args.video_bitrate is None):
-            cmd.extend(['-crf', args.constant_rate_factor])
-        else:
-            cmd.extend(['-b:v', args.video_bitrate])
+    cmd = fset(cmd, '-crf', args.constant_rate_factor)
+    cmd = fset(cmd, '-b:v', args.video_bitrate)
+    cmd = fset(cmd, '-tune', args.tune)
+    cmd = fset(cmd, '-preset', args.preset)
 
-    if(args.tune != 'none'):
-        cmd.extend(['-tune', args.tune])
-    cmd.extend(['-preset', args.preset, '-movflags', '+faststart', '-strict', '-2'])
+    cmd.extend(['-movflags', '+faststart', '-strict', '-2'])
     return cmd
 
+def scaleToSped(ffmpeg, ffprobe, vidFile, args, temp):
+    SCALE = f'{temp}{sep()}scale.mp4'
+    SPED = f'{temp}{sep()}spedup.mp4'
+
+    cmd = ['-i', SCALE]
+    cmd = properties(cmd, args, vidFile, ffprobe)
+    cmd.append(SPED)
+    check_errors = ffmpeg.pipe(cmd)
+
+    if('Error' in check_errors or 'failed' in check_errors):
+        cmd = ['-i', SCALE]
+        if('-allow_sw 1' in check_errors):
+            cmd.extend(['-allow_sw', '1'])
+
+        cmd = properties(cmd, args, vidFile, ffprobe)
+        cmd.append(SPED)
+        ffmpeg.run(cmd)
+
 
 def renderAv(ffmpeg, ffprobe, vidFile: str, args, chunks: list, speeds: list, fps,
-    temp, log):
+    has_vfr, temp, log):
     import av
 
     totalFrames = chunks[len(chunks) - 1][1]
     videoProgress = ProgressBar(totalFrames, 'Creating new video',
         args.machine_readable_progress, args.no_progress)
 
-    input_ = av.open(vidFile)
+    if(has_vfr):
+        class Wrapper:
+            """
+            Wrapper which only exposes the `read` method to avoid PyAV
+            trying to use `seek`.
+            From: github.com/PyAV-Org/PyAV/issues/578#issuecomment-621362337
+            """
+
+            name = "<wrapped>"
+
+            def __init__(self, fh):
+                self._fh = fh
+
+            def read(self, buf_size):
+                return self._fh.read(buf_size)
+
+        # Create a cfr stream on stdout.
+        cmd = ['-i', vidFile, '-map', '0:v:0', '-vf', f'fps=fps={fps}', '-r', str(fps),
+            '-vsync', '1', '-f', 'matroska', '-vcodec', 'rawvideo', 'pipe:1']
+
+        wrapper = Wrapper(ffmpeg.Popen(cmd).stdout)
+        input_ = av.open(wrapper, 'r')
+    else:
+        input_ = av.open(vidFile)
+
     inputVideoStream = input_.streams.video[0]
     inputVideoStream.thread_type = 'AUTO'
 
@@ -43,15 +93,16 @@ def renderAv(ffmpeg, ffprobe, vidFile: str, args, chunks: list, speeds: list, fp
 
     log.debug(f'   - pix_fmt: {pix_fmt}')
 
-    cmd = [ffmpeg.getPath(), '-y', '-f', 'rawvideo', '-vcodec', 'rawvideo', '-pix_fmt',
-        pix_fmt, '-s', f'{width}*{height}', '-framerate', f'{fps}', '-i', '-', '-pix_fmt',
-        pix_fmt]
+    cmd = [ffmpeg.getPath(), '-hide_banner', '-y', '-f', 'rawvideo', '-vcodec', 'rawvideo',
+        '-pix_fmt', pix_fmt, '-s', f'{width}*{height}', '-framerate', f'{fps}', '-i', '-',
+        '-pix_fmt', pix_fmt]
 
     if(args.scale != 1):
-        cmd.extend(['-vf', f'scale=iw*{args.scale}:ih*{args.scale}'])
-
-    cmd = properties(cmd, args, vidFile, ffprobe)
-    cmd.append(f'{temp}{sep()}spedup.mp4')
+        cmd.extend(['-vf', f'scale=iw*{args.scale}:ih*{args.scale}',
+            f'{temp}{sep()}scale.mp4'])
+    else:
+        cmd = properties(cmd, args, vidFile, ffprobe)
+        cmd.append(f'{temp}{sep()}spedup.mp4')
 
     if(args.show_ffmpeg_debug):
         process2 = subprocess.Popen(cmd, stdin=subprocess.PIPE)
@@ -83,9 +134,13 @@ def renderAv(ffmpeg, ffprobe, vidFile: str, args, chunks: list, speeds: list, fp
         process2.stdin.close()
         process2.wait()
     except BrokenPipeError:
-        process2 = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.DEVNULL)
+        log.print(cmd)
+        process2 = subprocess.Popen(cmd, stdin=subprocess.PIPE)
         log.error('Broken Pipe Error!')
 
+    if(args.scale != 1):
+        scaleToSped(ffmpeg, ffprobe, vidFile, args, temp)
+
     if(log.is_debug):
         log.debug('Writing the output file.')
     else:
@@ -93,10 +148,19 @@ def renderAv(ffmpeg, ffprobe, vidFile: str, args, chunks: list, speeds: list, fp
 
 
 def renderOpencv(ffmpeg, ffprobe, vidFile: str, args, chunks: list, speeds: list, fps,
-    effects, temp, log):
+    has_vfr, effects, temp, log):
     import cv2
+    import numpy as np
+    from interpolate import interpolate
+
+    if(has_vfr):
+        cmd = ['-i', vidFile, '-map', '0:v:0', '-vf', f'fps=fps={fps}', '-r', str(fps),
+            '-vsync', '1', '-f','matroska', '-vcodec', 'rawvideo', 'pipe:1']
+        fileno = ffmpeg.Popen(cmd).stdout.fileno()
+        cap = cv2.VideoCapture('pipe:{}'.format(fileno))
+    else:
+        cap = cv2.VideoCapture(vidFile)
 
-    cap = cv2.VideoCapture(vidFile)
     width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
     height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
     fourcc = cv2.VideoWriter_fourcc(*'mp4v')
@@ -104,12 +168,15 @@ def renderOpencv(ffmpeg, ffprobe, vidFile: str, args, chunks: list, speeds: list
     if(args.scale != 1):
         width = int(width * args.scale)
         height = int(height * args.scale)
+        video_name = f'{temp}{sep()}scale.mp4'
+    else:
+        video_name = f'{temp}{sep()}spedup.mp4'
 
     if(width < 2 or height < 2):
         log.error('Resolution too small.')
-
     log.debug(f'\n Resolution {width}x{height}')
-    out = cv2.VideoWriter(f'{temp}/spedup.mp4', fourcc, fps, (width, height))
+
+    out = cv2.VideoWriter(video_name, fourcc, fps, (width, height))
 
     totalFrames = chunks[len(chunks) - 1][1]
     cframe = 0
@@ -138,9 +205,6 @@ def renderOpencv(ffmpeg, ffprobe, vidFile: str, args, chunks: list, speeds: list
         # cframe not in chunks
         return 0
 
-    import numpy as np
-    from interpolate import interpolate
-
     def values(val, log, _type, totalFrames, width, height):
         if(val == 'centerX'):
             return int(width / 2)
@@ -293,8 +357,7 @@ def renderOpencv(ffmpeg, ffprobe, vidFile: str, args, chunks: list, speeds: list
 
         if(effects == [] and args.scale != 1):
             inter = cv2.INTER_CUBIC if args.scale > 1 else cv2.INTER_AREA
-            frame = cv2.resize(frame, (width, height),
-                interpolation=inter)
+            frame = cv2.resize(frame, (width, height), interpolation=inter)
 
         cframe = int(cap.get(cv2.CAP_PROP_POS_FRAMES)) # current frame
 
@@ -316,10 +379,12 @@ def renderOpencv(ffmpeg, ffprobe, vidFile: str, args, chunks: list, speeds: list
     out.release()
     cv2.destroyAllWindows()
 
-    if(args.video_codec != 'uncompressed'):
+    if(args.scale == 1):
         cmd = properties(['-i', vidFile], args, vidFile, ffprobe)
-        cmd.append(f'{temp}/spedup.mp4')
+        cmd.append(f'{temp}{sep()}spedup.mp4')
         ffmpeg.run(cmd)
+    else:
+        scaleToSped(ffmpeg, ffprobe, vidFile, args, temp)
 
     if(log.is_debug):
         log.debug('Writing the output file.')
